#include <owl\owlpch.h>

#include "browse.h"
#include "test.rc"

#define COL_CODE     100
#define COL_COMPANY  101
#define COL_ADDRESS  102
#define COL_CITY     103

const MAXEDITLEN			= 31;
const WM_DIALOG_CLOSED	= WM_USER + 100;


//----------------------------------------------------------------------------

struct TTransferStruct {
	TTransferStruct();

	TBrowseData	BrowseData;
	char			Code[MAXEDITLEN];
	char			Company[MAXEDITLEN];
	char			Address[MAXEDITLEN];
	char			City[MAXEDITLEN];
	char			LBDataEdit[MAXEDITLEN];
};

TTransferStruct::TTransferStruct()
{
	Code[0]		= 0;
	Company[0]	= 0;
	Address[0]	= 0;
	City[0]		= 0;

	BrowseData.AddRow();
	BrowseData.SetColumnText(COL_CODE,     "INGE");
	BrowseData.SetColumnText(COL_COMPANY,  "INGE-COM");
	BrowseData.SetColumnText(COL_ADDRESS,  "132, rue Martre");
	BrowseData.SetColumnText(COL_CITY,     "92110 Clichy");

	BrowseData.AddRow(true);
	BrowseData.SetColumnText(COL_CODE,     "BOR");
	BrowseData.SetColumnText(COL_COMPANY,	"BORLAND");
	BrowseData.SetColumnText(COL_ADDRESS,  "12, rue du logiciel");
	BrowseData.SetColumnText(COL_CITY,     "75002 Paris");
}


//----------------------------------------------------------------------------

class TBrowseDlg : public TDialog
{
public:

	TBrowseDlg(TWindow* parent, TTransferStruct& ts);

protected:

	void AddNewRow();
	void CmOk();
	void DeleteRow();
	void EvSelChange();
	void GetColumnsText();
	void ModifyColumnsText();

private:

	TBrowse*	Browse;
	TEdit*	Code;
	TEdit*	Company;
	TEdit*	Address;
	TEdit*	City;

	DECLARE_RESPONSE_TABLE(TBrowseDlg);
};

DEFINE_RESPONSE_TABLE1(TBrowseDlg, TDialog)
	EV_LBN_SELCHANGE(IDC_BROWSE, EvSelChange),
	EV_COMMAND(IDC_GETCOLTEXT, GetColumnsText),
	EV_COMMAND(IDC_MODIFYCOLTEXT, ModifyColumnsText),
	EV_COMMAND(IDC_ADDROW, AddNewRow),
	EV_COMMAND(IDC_DELETEROW, DeleteRow),
	EV_COMMAND(IDOK, CmOk),
END_RESPONSE_TABLE;

TBrowseDlg::TBrowseDlg(TWindow* parent, TTransferStruct& ts)
	: TDialog(parent, IDD_DIALOG)
{
	Browse = new TBrowse(this, IDC_BROWSE);
	Browse->AddColumn(COL_CODE,		"Code",		-1, false);
	Browse->AddColumn(COL_COMPANY,	"Company",	15			);
	Browse->AddColumn(COL_ADDRESS,	"Address",	20			);
	Browse->AddColumn(COL_CITY,		"City"					);

	Code		= new TEdit(this, IDC_CODE,		sizeof(ts.Code)		);
	Company	= new TEdit(this, IDC_COMPANY,	sizeof(ts.Company)	);
	Address	= new TEdit(this, IDC_ADDRESS,	sizeof(ts.Address)	);
	City		= new TEdit(this, IDC_CITY,		sizeof(ts.City)		);

	SetTransferBuffer(&ts);
}

void TBrowseDlg::AddNewRow()
{
	char text[MAXEDITLEN];

	Browse->AddRow();

	Code->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_CODE, text);

	Company->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_COMPANY, text);

	Address->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_ADDRESS, text);

	City->GetText(text, sizeof(text));
	Browse->SetColumnText(COL_CITY, text);
}

void TBrowseDlg::DeleteRow()
{
	int index = Browse->GetSelIndex();
	if (Browse->GetSelIndex() == LB_ERR) {
		MessageBox("No row selected !", GetModule()->GetName(), MB_ICONSTOP | MB_OK);
		return;
	}

	Browse->DeleteRow(index);

	Code		->Clear();
	Company	->Clear();
	Address	->Clear();
	City		->Clear();
}

void TBrowseDlg::CmOk()
{
	TDialog::CmOk();
	Parent->PostMessage(WM_DIALOG_CLOSED);
}

void TBrowseDlg::EvSelChange()
{
	GetColumnsText();
}

void TBrowseDlg::GetColumnsText()
{
	int index = Browse->GetSelIndex();
	if (index != LB_ERR) {
		string str;

		Browse->GetColumnText(COL_CODE, str, index);
		Code->SetText(str.c_str());

		Browse->GetColumnText(COL_COMPANY, str, index);
		Company->SetText(str.c_str());

		Browse->GetColumnText(COL_ADDRESS, str, index);
		Address->SetText(str.c_str());

		Browse->GetColumnText(COL_CITY, str, index);
		City->SetText(str.c_str());
	}
}

void TBrowseDlg::ModifyColumnsText()
{
	int index = Browse->GetSelIndex();
	if (index == LB_ERR) {
		MessageBox("No row selected !", GetModule()->GetName(), MB_ICONSTOP | MB_OK);
		return;
	}

	char text[MAXEDITLEN];

	Code->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_CODE, text, index);

	Company->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_COMPANY, text, index);

	Address->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_ADDRESS, text, index);

	City->GetText(text, sizeof(text));
	Browse->ModifyColumnText(COL_CITY, text, index);
}


//----------------------------------------------------------------------------

class TColorsDlg : public TDialog
{
public:

	TColorsDlg(TWindow* parent, TBrowse* browse)
		: TDialog(parent, IDD_COLORS) {Browse = browse;}

protected:

	void CmAddress();
	void CmCity();
	void CmCode();
	void CmCompany();
	void SetColor(int id);

private:

	TBrowse* Browse;

	DECLARE_RESPONSE_TABLE(TColorsDlg);
};

DEFINE_RESPONSE_TABLE1(TColorsDlg, TDialog)
	EV_COMMAND(IDC_CODE, CmCode),
	EV_COMMAND(IDC_COMPANY, CmCompany),
	EV_COMMAND(IDC_ADDRESS, CmAddress),
	EV_COMMAND(IDC_CITY, CmCity),
END_RESPONSE_TABLE;

void TColorsDlg::CmAddress()
{
	SetColor(COL_ADDRESS);
}

void TColorsDlg::CmCity()
{
	SetColor(COL_CITY);
}

void TColorsDlg::CmCode()
{
	SetColor(COL_CODE);
}

void TColorsDlg::CmCompany()
{
	SetColor(COL_COMPANY);
}

void TColorsDlg::SetColor(int id)
{
	static TColor custColors[16];
	TChooseColorDialog::TData choose;

	choose.Flags = CC_RGBINIT | CC_PREVENTFULLOPEN;
	Browse->GetHeaderColor(id, choose.Color);
	choose.CustColors = custColors;

	if (TChooseColorDialog(this, choose).Execute() == IDOK)
		Browse->SetHeaderColor(id, choose.Color);
}


//----------------------------------------------------------------------------

class TBrowseWindow : TBrowse
{
public:

	TBrowseWindow();
};

TBrowseWindow::TBrowseWindow()
	: TBrowse(0, 0, 0, 0, 100, 100)
{
	AddColumn(COL_CODE,		"Code",		-1, false);
	AddColumn(COL_COMPANY,	"Company",	20			);
	AddColumn(COL_ADDRESS,	"Address",	30			);
	AddColumn(COL_CITY,		"City"					);
}


//----------------------------------------------------------------------------

class TFrame : public TFrameWindow
{
public:

	TFrame(const char far *title);

protected:

	void CeClear(TCommandEnabler& ce);
	void CeDelete(TCommandEnabler& ce);
	void CeHideCode(TCommandEnabler& ce);
	void CeShowCode(TCommandEnabler& ce);

	void CmClear();
	void CmDelete();
	void CmDialog();
	void CmHideCode();
   void CmSetColor();
	void CmShowCode();

	LRESULT EvDialogClosed(WPARAM wParam, LPARAM lParam);

private:

	TTransferStruct TransferStruct;

	DECLARE_RESPONSE_TABLE(TFrame);
};


DEFINE_RESPONSE_TABLE1(TFrame, TFrameWindow)
	EV_COMMAND(CM_DIALOG, CmDialog),
	EV_COMMAND(CM_EDITCLEAR, CmClear),
	EV_COMMAND(CM_EDITDELETE, CmDelete),
	EV_COMMAND(CM_SHOWCODE, CmShowCode),
	EV_COMMAND(CM_HIDECODE, CmHideCode),
   EV_COMMAND(CM_SETCOLOR, CmSetColor),
	EV_COMMAND_ENABLE(CM_EDITCLEAR, CeClear),
	EV_COMMAND_ENABLE(CM_EDITDELETE, CeDelete),
	EV_COMMAND_ENABLE(CM_SHOWCODE, CeShowCode),
	EV_COMMAND_ENABLE(CM_HIDECODE, CeHideCode),
	EV_MESSAGE(WM_DIALOG_CLOSED, EvDialogClosed),
END_RESPONSE_TABLE;

TFrame::TFrame(const char far *title)
	: TFrameWindow(0, title, (TWindow*)new TBrowseWindow)
{
	AssignMenu(IDM_MENU);
   Attr.AccelTable = IDM_MENU;
}

void TFrame::CeClear(TCommandEnabler& ce)
{
	ce.Enable(TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->GetCount() > 0);
}

void TFrame::CeDelete(TCommandEnabler& ce)
{
	ce.Enable(TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->GetSelIndex() != LB_ERR);
}

void TFrame::CeHideCode(TCommandEnabler& ce)
{
	ce.Enable(TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->IsColumnVisible(COL_CODE));
}

void TFrame::CeShowCode(TCommandEnabler& ce)
{
	ce.Enable(!TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->IsColumnVisible(COL_CODE));
}

void TFrame::CmClear()
{
	TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->ClearList();
}

void TFrame::CmDelete()
{
	TBrowse* browse = TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse);
	int index = browse->GetSelIndex();

	if (index != LB_ERR)
		browse->DeleteRow(index);
}

void TFrame::CmDialog()
{
//	TBrowseDlg(this, TransferStruct).Execute();

	static TBrowseDlg *dlg = 0;
	if (!dlg)
		dlg = new TBrowseDlg (this, TransferStruct);
	dlg->Execute ();
}

void TFrame::CmHideCode()
{
	TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->ShowColumn(COL_CODE, false);
}

void TFrame::CmSetColor()
{
	TColorsDlg(this, TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)).Execute();
}

void TFrame::CmShowCode()
{
	TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse)->ShowColumn(COL_CODE);
}

LRESULT TFrame::EvDialogClosed(WPARAM, LPARAM)
{
	if (MessageBox("Update the Browse with the contents\nof the closed dialog box ?",
					  "Transfer Buffer", MB_YESNO) == IDYES) {

		TBrowse* browse = TYPESAFE_DOWNCAST(GetClientWindow(), TBrowse);
		TBrowseData& browseData = TransferStruct.BrowseData;

      browse->ClearList();

		if (browseData.GetNumRows() > 0) {
			TRowArrayIterator iter(browseData.GetItemDatas());
			while (iter) {
				string code, comp, addr, city;
				iter.GetColumnText(COL_CODE,		code);
				iter.GetColumnText(COL_COMPANY,	comp);
				iter.GetColumnText(COL_ADDRESS,	addr);
				iter.GetColumnText(COL_CITY,		city);

				browse->AddRow();
				browse->SetColumnText(COL_CODE,		code.c_str());
				browse->SetColumnText(COL_COMPANY,	comp.c_str());
				browse->SetColumnText(COL_ADDRESS,	addr.c_str());
				browse->SetColumnText(COL_CITY,		city.c_str());

				iter++;
			}
		}
	}

	return 1;
}


//----------------------------------------------------------------------------

class TApp : public TApplication
{
public:

	TApp() : TApplication("TBrowse 2.1  Daniel BERMAN") {}

protected:

	void InitMainWindow() {SetMainWindow(new TFrame(GetName()));}
};


//----------------------------------------------------------------------------

int OwlMain(int, char*[])
{
	return TApp().Run();
}
